;================================================================
; Project:		NSIS Installer Core
; Installer Author:	Bob Ward  (AOL Dulles)
;
; Copyright 2005 -- America Online Inc
;================================================================

;--------------------------------
;Compression settings

        SetCompressor lzma
        SetCompressorDictSize 16		; Adjust for optimum compressed EXE size (size is in MB)

;--------------------------------
;Extra Paths for Includes and Plugins

        ; Includes
        !addincludedir "..\..\..\..\memberacq\components\common\include"

        ; Plugins
        !addplugindir "..\..\..\..\memberacq\components\common\plugins\bin"

;--------------------------------
; Defines...Constants...Variables

        !include "defines_consts_vars.nsh"

        !define PRODUCT_UNINST_REGKEY "Software\Microsoft\Windows\CurrentVersion\Uninstall"
        
        ; Installer Path Variables -- Initialize these in the ".onInit" and "un.onInit" callback functions
        Var COM_APP_DATA   ; Common Application Data Folder (eg: "C:\Documents and Settings\All Users")

        ;Diskspace
        Var TOTALSPACE          ; Amount of total diskspace required
        Var SYSSPACE		; Amount of diskspace required on System Drive in Kilobytes
        Var ALTSPACE		; Amount of diskspace required on non-System Drive in Kilobytes
        Var TMPSPACE		; Amount of temporary diskspace required to run installer in Kilobytes

        ;Mutex Variables
        Var CHECK_MUTX          ; Holds whether or not we need to check the mutex
        
        ;Message Boxes
        Var POP_MSG_BOX         ; Holds whether or not we need to pop a message box
        
        ;Configuration File
        Var INI_PATH
        Var SECTION
        Var DISK_SPACE
        Var DO_ADDREMOVE_ENTRY
        Var PRODUCT_UNINSTSTR
        
               
        ;OCP EE Dir
        Var EE_DIR              ; Directory for the OCP EE

;--------------------------------
;Includes

        !include "InstallFile.nsh"      ; File install macros for replacing by version and replacing in-use files
        !include "reftrackapi.nsh"      ; NSIS helper functions for calling RefTrack API
        !include "utility.nsh"          ; AOL Utility Plugin
	
        !include ".\${_LOCALE_}\stringtable.nsh"		; localized string resources and constants

;--------------------------------
; Build and Install Settings

	AllowRootDirInstall false
	
	SilentInstall silent
	SilentUninstall silent
	
  ;Name and file
  Name "${PRODUCT_NAME} ${_PRODUCT_MAJORVER_}"
  OutFile "${_EXENAME_}"

;--------------------------------
; Version Information Block
        !include "version_info.nsh"

;--------------------------------
; Special Support Macros -- Custom user features

        !include "functions.nsh"

;--------------------------------
; InstallTypes -- Each becomes an option in the MUI_PAGE_COMPONENTS dialog

InstType "Typical"
InstType /NOCUSTOM

!define MAXINSTTYPES 1

;--------------------------------
; Installation - This is where you write your installer

Section "S_Main" Sec_Main_Sys
SectionIn 1 RO    ; Typical install

    ; ** Call the utility plugin from inside a function
    ; ** to make available in the HFE
		utility::IsInstDriverSyncMutxOpen /NOUNLOAD
		Pop $0

	; Turn Install Logging ON 
	LogSet on
        LogText "*** Start Log ***"
  
  ; Update the exit code
  WriteIniStr $INI_PATH $SECTION "ExitCode" "1"

	; Initialize the progress bar
	Push "1"
	Call UpdateProgress

!ifndef LANG_PACK        
  ;Create uninstaller (Needed for Rollback so install it first)
  CreateDirectory "$INSTDIR"		; Ensure destination folder exists before using it
  WriteUninstaller "$INSTDIR\uninst.exe"

  ; Create Add/Remove entry for the app, if applicable
  StrCmp $DO_ADDREMOVE_ENTRY "Y" 0 skipOSAddRemovePrograms1
  	; Add entry to Add/Remove Programs only when uberuninstaller absent
  	IfFileExists "$COMMONFILES\AOL\Uninstaller.exe" skipOSAddRemovePrograms1
  		; Uberuninstaller not present -- Force Add/Remove Programs Entry
  		StrCpy $DO_ADDREMOVE_ENTRY "A"
  skipOSAddRemovePrograms1:
  
  StrCmp $DO_ADDREMOVE_ENTRY "A" 0 skipOSAddRemovePrograms2
		WriteRegStr HKLM "${PRODUCT_UNINST_REGKEY}\${PRODUCT_NAME} ${_PRODUCT_MAJORVER_}" "DisplayName" "${PRODUCT_UNINSTSTRING}"
		WriteRegStr HKLM "${PRODUCT_UNINST_REGKEY}\${PRODUCT_NAME} ${_PRODUCT_MAJORVER_}" "DisplayIcon" "$COMMONFILES\AOL\AOLLaunch.exe"
		WriteRegStr HKLM "${PRODUCT_UNINST_REGKEY}\${PRODUCT_NAME} ${_PRODUCT_MAJORVER_}" "UninstallString" "$INSTDIR\uninst.exe"
  skipOSAddRemovePrograms2:

	; ********************************
	; Prepare for Reference Tracking and Rollback support
	; ********************************
	!include "references.nsh"
  ; **** Reference Tracking Support ****
!endif

	; ********************************
	; Begin Installation
	; ********************************
	
	SetOutPath "$INSTDIR"   ; Set installation working folder to selected installation folder

; --> Start your installation/upgrade process here <--

!ifndef LANG_PACK        
	; **** Store installation folder ****
	WriteRegStr HKLM "${REGKEYMAIN}" "InstallDir" $INSTDIR
!endif

SectionEnd

;--------------------------------
; Fileset - This is where the unique fileset is included to be installed

        !include "fileset.nsh"

Section "S_Cleanup" Cleanup
SectionIn 1 RO    ; Typical install

SectionEnd

!ifndef LANG_PACK        
;--------------------------------
; Uninstallation - This is where you write your uninstaller

Section "Uninstall" Sec_Uninstall_Main
	
;--------------------------------
; Uninstallation - This is where you include your uninstaller
        !include "uninst.nsh"


!ifndef LANG_PACK        
	; **** Remove installation folder ****
	DeleteRegValue HKLM "${REGKEYMAIN}" "InstallDir"
	DeleteRegKey /ifempty HKLM "${REGKEYMAIN}"	
!ifdef REGKEYPROD
	DeleteRegKey /ifempty HKLM "${REGKEYPROD}"	
!endif
!endif
        	
SectionEnd
!endif

;================================
; Functions
;================================

;--------------------------------
; Installation Callbacks -- Called automatically by NSIS installer

Function .onInit
        ;Determine EE Dir
        ReadRegStr "$EE_DIR" HKLM "SOFTWARE\America Online\OCP" "InstallDir"

        ;Default installation folder
        StrCpy $INSTDIR "${INSTALL_DIRECTORY}"
        
        ;RDW -- REMOVING FOR NOW.....        
        ;;Overwrite with dir found in registry if exists
        ;ReadRegStr $R0 HKLM "${REGKEYMAIN}" "InstallDir"
        ;StrCmp $R0 "" reg_install_dir_not_found 0
        ;StrCpy $INSTDIR $R0
        ;reg_install_dir_not_found:

	; ****** Initialize Uninstaller Path Variables ******
	ReadRegStr $COM_APP_DATA HKLM "Software\Microsoft\Windows\CurrentVersion\explorer\Shell Folders" "Common AppData"
	StrCmp $COM_APP_DATA "" 0 endCOM_APP_DATA
		StrCpy $COM_APP_DATA "$WINDIR\All Users\Application Data"   ; Default if all else fails
	endCOM_APP_DATA:

  Push "IniPath"
  Push ""
  Call GetParameterValue
  Pop $INI_PATH
  
  Push "section"
  Push ""
  Call GetParameterValue
  Pop $SECTION

  Push "ds"
  Push ""
  Call GetParameterValue
  Pop $DISK_SPACE

  StrCmp "$DISK_SPACE" "Y" 0 skipWriteDiskSpace
	  ; A diskspace query requires a valid Installer Driver config file -- Ensure it exists
		IfFileExists "$INI_PATH" endIniExistChk2
  		Abort		; Fatal Error -- Configuration file missing
  	endIniExistChk2:
  
		StrCmp $SECTION "" 0 endSectionChk2
  		Abort		; Fatal Error -- Do not know what section to use
  	endSectionChk2:
  
	  ; Get the size of the installer
	  Call CalcReqDiskspace
  	;SectionGetSize ${Sec_Main} $0
  
	  ; Update the ini with the disk space required
	  WriteIniStr "$INI_PATH" $SECTION "SysSize" "$SYSSPACE"
	  WriteIniStr "$INI_PATH" $SECTION "AltSize" "$ALTSPACE"
	  WriteIniStr "$INI_PATH" $SECTION "TmpSize" "$TMPSPACE"
	  
          ; Update the exit code
          WriteIniStr $INI_PATH $SECTION "ExitCode" "2"

	  Quit
	skipWriteDiskSpace:

	; Determine Uninstaller Display string
	IfFileExists "$INI_PATH" elseIniExistChk1 0
		; Assume running standalone -- Use defaults
 		StrCpy $DO_ADDREMOVE_ENTRY "Y"
 		StrCpy $PRODUCT_UNINSTSTR "${PRODUCT_UNINSTSTRING}"
 		Goto endIniExistChk1
 	elseIniExistChk1:
 		; INI file exists
		StrCmp $SECTION "" 0 endSectionChk1
  		Abort		; Fatal Error -- Do not know what section to use
  	endSectionChk1:
  
  	; Use INI defined values
  	ReadIniStr $PRODUCT_UNINSTSTR $INI_PATH $SECTION "AOLUninstStr"
  	ReadIniStr $DO_ADDREMOVE_ENTRY $INI_PATH $SECTION "OSAddRemove"
 	endIniExistChk1:
FunctionEnd

Function .onInstSuccess
  
  ; Update the exit code
  WriteIniStr $INI_PATH $SECTION "ExitCode" "0"

	IfRebootFlag 0 endRebootFlagChk
		SetRebootFlag false	; suppress reboots
		
		Push ${APP_ID_USED_BY_PROGUPD}
		Call SetReboot
		Pop $0		; ALWAYS pop return code to prevent stack overflows
	endRebootFlagChk:
	
	; Ensure that 100% is reported for a successful install
	Push "100"
	Call UpdateProgress

FunctionEnd

Function .onInstFailed

; User has cancelled or an exit via "Abort" occurred
        
	SetRebootFlag false	; suppress reboots

FunctionEnd

; *** Special Callbacks ***
; These may return allowing the install to continue
; -- Ensure that the values of any registers ($0 to $9 and $R0 to $R9) are preserved

Function .onUserAbort
; Use this callback if you want to capture and prompt when user's select "Cancel"

	MessageBox MB_YESNO "Abort install?" IDYES NoCancelAbort
		Abort ; causes installer to not quit.
	NoCancelAbort:
FunctionEnd

Function .onVerifyInstDir
	; Used to verify the validity of the selected installation folder
	
	IfFileExists "$INSTDIR\Winamp.exe" PathGood
		Abort ; if $INSTDIR is not a winamp directory, don't let us install there
	PathGood:
FunctionEnd


;--------------------------------
; Uninstallation Callbacks -- Called automatically by NSIS uninstaller

Function un.onInit
        
  ;Determine EE Dir
  ReadRegStr "$EE_DIR" HKLM "SOFTWARE\America Online\OCP" "InstallDir"

  ; Open Uninstaller Mutex
  utility::OpenUninstallerSyncMutx /NOUNLOAD
  Pop $0

	; ****** Initialize Uninstaller Path Variables ******
	ReadRegStr $COM_APP_DATA HKLM "Software\Microsoft\Windows\CurrentVersion\explorer\Shell Folders" "Common AppData"
	StrCmp $COM_APP_DATA "" 0 endCOM_APP_DATA
		StrCpy $COM_APP_DATA "$WINDIR\All Users\Application Data"   ; Default if all else fails
	endCOM_APP_DATA:

	; ****** Read Command Line ******
  ; Check cmd line param to see if we need to check mutex
  Push "CheckMutx"
  Push ""
  Call un.GetParameterValue
  Pop $CHECK_MUTX
 
  StrCmp "$CHECK_MUTX" "N" skipMutxCheck 0
		; The following plug-in function to check if the mutex is open
		; actually opens the mutex.  So there is no need to explictly
		; open it again.
		utility::IsInstDriverSyncMutxOpen /NOUNLOAD
		Pop $0
		StrCmp $0 "1" mutexNotAlreadyOpen
			MessageBox MB_OK "${ERR_UNINSTCONFLICT}"	; This string is Localizable
			Abort
		mutexNotAlreadyOpen:
	skipMutxCheck:

  Push "PopUpMsgBox"
  Push ""
  Call un.GetParameterValue
  Pop $POP_MSG_BOX

	; ****** Check for Correct Permissions ******
  UserInfo::GetAccountType
  Pop $0
  StrCmp "$0" "Admin" continueUninstall 0
    MessageBox MB_OK "${MSG_NO_ADMIN}"		; This string is Localizable
		Abort
  continueUninstall:

FunctionEnd

Function un.onUninstSuccess
  StrCmp "$POP_MSG_BOX" "N" skipMsgBox 0
	  MessageBox MB_ICONINFORMATION|MB_OK "${MSG_UNINST_SUCC}"
  skipMsgBox:
  
  ; Set RunOnce entry so other AOL apps know not to install/uninstall if a reboot is pending
  ; (only necessary if Uber Uninstaller not running, otherwise send reboot message)
  IfRebootFlag 0 noReboot
    FindWindow $0 "AOLUberUninstaller"
    IntCmp $0 0 noUberUninstaller
      SendMessage $0 1283 0 0  ; 1283 = WM_USER+259 = Reboot needed
      Goto noReboot
    noUberUninstaller:
    ; Uninstaller running standalone, so write the run once key
    WriteRegStr HKLM "SOFTWARE\Microsoft\Windows\CurrentVersion\RunOnce" "AOLRebootNeeded" "regsvr32.exe /s"
  noReboot:
FunctionEnd

Function un.onUninstFailed

; User has cancelled or an exit via "Abort" occurred

	SetRebootFlag false	; suppress reboots

FunctionEnd

; *** Special Callbacks ***
; These may return allowing the uninstall to continue
; -- Ensure that the values of any registers ($0 to $9 and $R0 to $R9) are preserved

Function un.onUserAbort
; Use this callback if you want to capture and prompt when users select "Cancel"

	MessageBox MB_YESNO "Abort install?" IDYES NoCancelAbort
		Abort ; causes installer to not quit.
	NoCancelAbort:
FunctionEnd


;--------------------------------
; Special Support Functions -- Custom user functions

; **** Installer Support ****
; Function to update progress
Function        UpdateProgress
  Exch $1  ; Pop PercentDone off stack and push current contents of $1 on stack
  Push $0  ; $0 used for AppID in this function

  IfFileExists "$TEMP\ProgUpd.dll" 0 noProgDLLPresent
    StrCpy $0 "${APP_ID_USED_BY_PROGUPD}"
  	System::Call '$TEMP\ProgUpd::StdUpdateProgress(t r0, i r1) i.r2'
	noProgDLLPresent:

  ; Restore previous values of $0 and $1 for calling routine
  Pop $0
  Pop $1
FunctionEnd

; Function to set reboot
Function        SetReboot
  Push $0  ; $0 used for AppID in this function

  IfFileExists "$TEMP\ProgUpd.dll" 0 noProgDLLPresent
    StrCpy $0 "${APP_ID_USED_BY_PROGUPD}"
  	System::Call '$TEMP\ProgUpd::SetReboot(t r0) i.r1'
	noProgDLLPresent:

  ; Restore previous values of $0 and $1 for calling routine
  Pop $0
FunctionEnd

Function	CalcReqDiskspace

	Push $0		; Index of Section to Query
	Push $1		; Section Name
	Push $2		; Temporary variable
	Push $3		; Diskspace required for section in K

	ClearErrors
	 
	StrCpy $0 "0"
loop:
	StrCpy $1 ""
	StrCpy $2 ""
	StrCpy $3 ""
	SectionGetText $0 $1
	IfErrors endDiskspaceScan
	StrCmp $1 "" continue

	ClearErrors
	 
	; Check if section is selected
	SectionGetFlags $0 $2
	IntOp $2 $2 & 1
	IntCmp $2 1 0 continue continue

	; Check where to apply diskspace for section
	StrCpy $2 $1 2 0
	StrCmp $2 "S_" checkSysSpace 0
	StrCmp $2 "A_" checkAltSpace 0
	StrCmp $2 "T_" checkTmpSpace 0
	goto continue		; Section with improper name found
	
checkSysSpace:
	SectionGetSize $0 $3
	IntOp $SYSSPACE $SYSSPACE + $3
	goto fixName
checkTmpSpace:
	SectionGetSize $0 $3
	IntCmp $TMPSPACE $3 fixName 0 fixName		; Store only largest temp space found
	StrCpy "$TMPSPACE" "$3"
	goto fixName
checkAltSpace:
	SectionGetSize $0 $3
	IntOp $ALTSPACE $ALTSPACE + $3
	
fixName:
	SectionSetText $0 ""		; Make section hidden in components dialog
continue:
;	MessageBox MB_OK "Requirements for '$0' -> '$3'"
	IntOp $0 $0 + 1
	goto loop
endDiskspaceScan:
;MessageBox MB_OK "Sys :<$SYSSPACE> Alt : <$ALTSPACE>   Tmp:  <$TMPSPACE>"
	; Find Total Diskspace to be installed for progress reporting
	IntOp $TOTALSPACE $SYSSPACE + $TMPSPACE
	IntOp $TOTALSPACE $TOTALSPACE + $ALTSPACE

	; Uncomment the following line for debugging purposes
;	MessageBox MB_OK "Total Diskspace Requirements -> Sys='$SYSSPACE'; Tmp='$TMPSPACE'; Alt='$ALTSPACE'; Total='$TOTALSPACE'"

	Pop $3
	Pop $2
	Pop $1
	Pop $0
	
FunctionEnd

!ifndef LANG_PACK        
; **** Uninstaller Support ****

Function	un.CalcReqDiskspace

	; Add code to query all active uninstall sections for their uninstallation diskspace

FunctionEnd
!endif




